var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { CharStreams, CommonTokenStream, } from 'antlr4ng';
import * as c3 from 'antlr4-c3';
import { TableSymbol } from './lib/symbolTable.js';
import { SqlErrorListener } from './lib/sqlErrorListener.js';
import { getCurrentStatement, shouldSuggestTemplates } from './lib/query.js';
import { findCursorTokenIndex, getCursorIndex } from './lib/cursor.js';
import { getTableQueryPosition } from './lib/tables.js';
import { mySqlParserData } from './parsers/mysql/mySqlParserData.js';
import { postgreSqlParserData } from './parsers/postgresql/postgreSqlParserData.js';
function parseQueryWithoutCursor(Lexer, Parser, tokenDictionary, getParseTree, query) {
    const inputStream = CharStreams.fromString(query);
    const lexer = new Lexer(inputStream);
    const tokenStream = new CommonTokenStream(lexer);
    const parser = new Parser(tokenStream);
    const errorListener = new SqlErrorListener(tokenDictionary.SPACE);
    parser.removeErrorListeners();
    parser.addErrorListener(errorListener);
    getParseTree(parser);
    return { errors: errorListener.errors };
}
function getColumnSuggestions(Lexer, Parser, SymbolTableVisitor, tokenDictionary, explicitlyParseJoin, getParseTree, initialTokenStream, cursor, currentStatement) {
    const realCursorTokenIndex = findCursorTokenIndex(initialTokenStream, cursor, tokenDictionary.SPACE, true);
    if (!realCursorTokenIndex) {
        throw new Error(`Could not find realCursorTokenIndex at Ln ${cursor.line}, Col ${cursor.column}`);
    }
    const tableQueryPosition = getTableQueryPosition(initialTokenStream, realCursorTokenIndex, tokenDictionary);
    if (tableQueryPosition) {
        const query = currentStatement.slice(tableQueryPosition.start, tableQueryPosition.end);
        const inputStream = CharStreams.fromString(query);
        const lexer = new Lexer(inputStream);
        const tokenStream = new CommonTokenStream(lexer);
        const parser = new Parser(tokenStream);
        parser.removeErrorListeners();
        const parseTree = getParseTree(parser, tableQueryPosition.type);
        const visitor = new SymbolTableVisitor();
        visitor.visit(parseTree);
        if (explicitlyParseJoin && tableQueryPosition.joinTableQueryPosition) {
            const joinTableQuery = currentStatement.slice(tableQueryPosition.joinTableQueryPosition.start, tableQueryPosition.joinTableQueryPosition.end);
            const joinInputStream = CharStreams.fromString(joinTableQuery);
            const joinLexer = new Lexer(joinInputStream);
            const joinTokenStream = new CommonTokenStream(joinLexer);
            const joinParser = new Parser(joinTokenStream);
            joinParser.removeErrorListeners();
            const joinParseTree = getParseTree(joinParser, 'from');
            visitor.visit(joinParseTree);
        }
        const tables = visitor.symbolTable.getNestedSymbolsOfTypeSync(TableSymbol);
        if (tables.length) {
            return {
                tables: tables.map((tableSymbol) => ({
                    name: tableSymbol.name,
                    alias: tableSymbol.alias,
                })),
            };
        }
    }
    return undefined;
}
const quotesRegex = /^'(.*)'$/;
export function parseQuery(Lexer, Parser, SymbolTableVisitor, tokenDictionary, ignoredTokens, preferredRules, explicitlyParseJoin, getParseTree, generateSuggestionsFromRules, query, cursor) {
    const inputStream = CharStreams.fromString(query);
    const lexer = new Lexer(inputStream);
    const tokenStream = new CommonTokenStream(lexer);
    const parser = new Parser(tokenStream);
    const errorListener = new SqlErrorListener(tokenDictionary.SPACE);
    parser.removeErrorListeners();
    parser.addErrorListener(errorListener);
    getParseTree(parser);
    const core = new c3.CodeCompletionCore(parser);
    core.ignoredTokens = ignoredTokens;
    const cursorTokenIndex = findCursorTokenIndex(tokenStream, cursor, tokenDictionary.SPACE);
    const suggestKeywords = [];
    let result = {
        errors: errorListener.errors,
    };
    if (cursorTokenIndex !== undefined) {
        // Subtracting 2, because of whitespace token
        const previousToken = tokenStream.get(cursorTokenIndex - 2);
        const { tokens } = core.collectCandidates(cursorTokenIndex);
        // When c3 comes across a preferred rule, it doesn't suggest tokens inside that rule in
        // tokens map (suggestKeywords), that's why we need to collect candidates for rules separately
        const coreForRules = new c3.CodeCompletionCore(parser);
        coreForRules.preferredRules = preferredRules;
        const { rules } = coreForRules.collectCandidates(cursorTokenIndex);
        const _a = generateSuggestionsFromRules(rules, cursorTokenIndex, previousToken), { suggestColumns } = _a, suggestionsFromRules = __rest(_a, ["suggestColumns"]);
        result = Object.assign(Object.assign({}, result), suggestionsFromRules);
        tokens.forEach((_, tokenType) => {
            var _a;
            // Literal keyword names are quoted
            const name = (_a = parser.vocabulary.getLiteralName(tokenType)) === null || _a === void 0 ? void 0 : _a.replace(quotesRegex, '$1');
            if (name) {
                suggestKeywords.push({
                    value: name,
                });
            }
        });
        const cursorIndex = getCursorIndex(query, cursor);
        const currentStatement = getCurrentStatement(query, cursorIndex);
        if (suggestColumns) {
            result.suggestColumns = getColumnSuggestions(Lexer, Parser, SymbolTableVisitor, tokenDictionary, explicitlyParseJoin, getParseTree, tokenStream, cursor, currentStatement.statement);
        }
        result.suggestTemplates = shouldSuggestTemplates(currentStatement.statement, currentStatement.cursorIndex);
    }
    result.suggestKeywords = suggestKeywords;
    return result;
}
export function parseMySqlQueryWithoutCursor(query) {
    return parseQueryWithoutCursor(mySqlParserData.Lexer, mySqlParserData.Parser, mySqlParserData.tokenDictionary, mySqlParserData.getParseTree, query);
}
export function parseMySqlQuery(query, cursor) {
    return parseQuery(mySqlParserData.Lexer, mySqlParserData.Parser, mySqlParserData.SymbolTableVisitor, mySqlParserData.tokenDictionary, mySqlParserData.ignoredTokens, mySqlParserData.preferredRules, mySqlParserData.explicitlyParseJoin, mySqlParserData.getParseTree, mySqlParserData.generateSuggestionsFromRules, query, cursor);
}
export function parsePostgreSqlQueryWithoutCursor(query) {
    return parseQueryWithoutCursor(postgreSqlParserData.Lexer, postgreSqlParserData.Parser, postgreSqlParserData.tokenDictionary, postgreSqlParserData.getParseTree, query);
}
export function parsePostgreSqlQuery(query, cursor) {
    return parseQuery(postgreSqlParserData.Lexer, postgreSqlParserData.Parser, postgreSqlParserData.SymbolTableVisitor, postgreSqlParserData.tokenDictionary, postgreSqlParserData.ignoredTokens, postgreSqlParserData.preferredRules, postgreSqlParserData.explicitlyParseJoin, postgreSqlParserData.getParseTree, postgreSqlParserData.generateSuggestionsFromRules, query, cursor);
}
