function getClosingBracketIndex(tokenStream, tokenIndex, dictionary) {
    let currentIndex = tokenIndex;
    while (currentIndex < tokenStream.size) {
        const token = tokenStream.get(currentIndex);
        if (token.type === dictionary.CLOSING_BRACKET) {
            return { cursorIndex: token.start, tokenIndex: currentIndex };
        }
        if (token.type === dictionary.OPENING_BRACKET) {
            return undefined;
        }
        currentIndex++;
    }
    const lastIndex = tokenStream.size - 1;
    return { cursorIndex: tokenStream.get(lastIndex).start, tokenIndex: lastIndex };
}
export function getTableQueryPosition(tokenStream, tokenIndex, dictionary) {
    const end = tokenStream.get(tokenStream.size - 1).start;
    let currentIndex = tokenIndex;
    let isAscending = false;
    // Go backward at first
    while (currentIndex >= 0 && currentIndex < tokenStream.size) {
        const token = tokenStream.get(currentIndex);
        // We don't want to check nested statement
        if (token.type === dictionary.OPENING_BRACKET ||
            token.type === dictionary.CLOSING_BRACKET) {
            if (isAscending) {
                break;
            }
            else {
                currentIndex = tokenIndex;
                isAscending = true;
            }
        }
        if (token.type === dictionary.FROM) {
            const closingBracketIndex = getClosingBracketIndex(tokenStream, tokenIndex, dictionary);
            if (!closingBracketIndex) {
                break;
            }
            const joinIndex = getJoinIndex(tokenStream, currentIndex, closingBracketIndex.tokenIndex, dictionary);
            const joinTableQueryPosition = joinIndex
                ? {
                    start: joinIndex,
                    end: closingBracketIndex.cursorIndex,
                }
                : undefined;
            return {
                start: token.start,
                end: closingBracketIndex.cursorIndex,
                type: 'from',
                joinTableQueryPosition,
            };
        }
        if (isAscending) {
            currentIndex++;
        }
        else {
            currentIndex--;
        }
        // Go forward if nothing is found
        if (currentIndex === -1) {
            currentIndex = tokenIndex;
            isAscending = true;
        }
    }
    // Could not find FROM, so we look for other keywords
    currentIndex = tokenIndex;
    while (currentIndex >= 0) {
        const token = tokenStream.get(currentIndex);
        // Doesn't work for now because suggestColumns is false for this case
        // In MySqlParser this is not fullColumnName but is uid
        // and also higher order rules (e.g. alterStatement) have precedence
        if (token.type === dictionary.ALTER) {
            return {
                start: token.start,
                end,
                type: 'alter',
            };
        }
        if (token.type === dictionary.INSERT) {
            return {
                start: token.start,
                end,
                type: 'insert',
            };
        }
        if (token.type === dictionary.UPDATE) {
            return {
                start: token.start,
                end,
                type: 'update',
            };
        }
        currentIndex--;
    }
    return undefined;
}
function getJoinIndex(tokenStream, startIndex, endIndex, dictionary) {
    let currentIndex = startIndex;
    while (currentIndex < endIndex) {
        const token = tokenStream.get(currentIndex);
        if (token.type === dictionary.JOIN) {
            return token.stop + 1;
        }
        currentIndex++;
    }
    return undefined;
}
