import { Lexer } from 'antlr4ng';
const letterRegex = /^\p{L}+$/u;
const integralType = 658;
const lessLessType = 18;
const greaterGreaterType = 19;
class PostgreSqlLexerBase extends Lexer {
    constructor() {
        super(...arguments);
        this.tags = [];
    }
    pushTag() {
        this.tags.push(this.text);
    }
    isTag() {
        return this.text === this.tags[0];
    }
    popTag() {
        this.tags.shift();
    }
    checkLA(c) {
        return this.inputStream.LA(1) !== c;
    }
    charIsLetter() {
        // TODO
        // @ts-ignore
        return letterRegex.test(this.inputStream.LA(-1));
    }
    HandleNumericFail() {
        this.inputStream.seek(this.inputStream.index - 2);
        this.type = integralType;
    }
    HandleLessLessGreaterGreater() {
        if (this.text == '<<') {
            this.type = lessLessType;
        }
        else if (this.text == '>>') {
            this.type = greaterGreaterType;
        }
    }
    UnterminatedBlockCommentDebugAssert() {
        // VOID
    }
    CheckIfUtf32Letter() {
        let codePoint = this.inputStream.LA(-2) << (8 + this.inputStream.LA(-1));
        let c;
        if (codePoint < 0x10000) {
            c = [codePoint];
        }
        else {
            codePoint -= 0x10000;
            c = [codePoint / 0x400 + 0xd800, (codePoint % 0x400) + 0xdc00];
        }
        return letterRegex.test(`${c[0]}`);
    }
}
export default PostgreSqlLexerBase;
